# -*- coding: utf-8 -*-

"""
.. module:: cim.v1.typeset_for_software_package.py

   :license: GPL / CeCILL
   :platform: Unix, Windows
   :synopsis: The set of types of the cim.v1.software package.

.. moduleauthor:: Earth System Documentation (ES-DOC) <dev@es-doc.org>
.. note:: Code generated using the pyesdoc framework.

"""
import abc
import datetime
import uuid

import typeset_for_shared_package as shared



class Component(shared.DataSource):
    """A concrete class within the cim v1 type system.

    A SofwareComponent is an abstract component from which all other components derive. It represents an element that takes input data and generates output data. A SoftwareCompnent can include nested "child" components. Every component can have "componentProperties" which describe the scientific properties that a component simulates (for example, temperature, pressure, etc.) and the numerical properties that influence how a component performs its simulation (for example, the force of gravity). A SoftwareComponent can also have a Deployment, which describes how software is deployed onto computing resources. And a SoftwareComponent can have a composition, which describes how ComponentProperties are coupled together either to/from other SoftwareComponents or external data files. The properties specified by a component's composition must be owned by that component or a child of that component; child components cannot couple together their parents' properties.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Component, self).__init__()

        self.citations = []                               # shared.Citation (0.N)
        self.code_access = None                           # unicode (0.1)
        self.composition = None                           # software.Composition (0.1)
        self.coupling_framework = None                    # software.CouplingFrameworkType (0.1)
        self.dependencies = []                            # software.EntryPoint (0.N)
        self.deployments = []                             # software.Deployment (0.N)
        self.description = None                           # unicode (0.1)
        self.funding_sources = []                         # unicode (0.N)
        self.grid = None                                  # grids.GridSpec (0.1)
        self.is_embedded = None                           # bool (0.1)
        self.language = None                              # software.ComponentLanguage (0.1)
        self.license = None                               # shared.License (0.1)
        self.long_name = None                             # unicode (0.1)
        self.online_resource = None                       # unicode (0.1)
        self.parent = None                                # software.Component (0.1)
        self.previous_version = None                      # unicode (0.1)
        self.properties = []                              # software.ComponentProperty (0.N)
        self.release_date = None                          # datetime.datetime (0.1)
        self.responsible_parties = []                     # shared.ResponsibleParty (0.N)
        self.short_name = None                            # unicode (1.1)
        self.sub_components = []                          # software.Component (0.N)
        self.version = None                               # unicode (0.1)

    def __str__(self):
        return "{}".format(self.short_name)


class ComponentLanguage(object):
    """A concrete class within the cim v1 type system.

    Details of the programming language a component is written in. There is an assumption that all EntryPoints use the same ComponentLanguage.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ComponentLanguage, self).__init__()

        self.name = None                                  # unicode (1.1)
        self.properties = []                              # software.ComponentLanguageProperty (0.N)


class ComponentLanguageProperty(shared.Property):
    """A concrete class within the cim v1 type system.

    This provides a place to include language-specific information. Every property is basically a name/value pair, where the names are things like: moduleName, reservedUnits, reservedNames (these are all examples of Fortran-specific properties).

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ComponentLanguageProperty, self).__init__()



class ComponentProperty(shared.DataSource):
    """A concrete class within the cim v1 type system.

    ComponentProperties include things that a component simulates (ie: pressure, humidity) and things that prescribe that simulation (ie: gravity, choice of advection scheme). Note that this is a specialisation of shared::DataSource. data::DataObject is also a specialisation of shared::DataSource. This allows software::Connections and/or activity::Conformance to refer to either ComponentProperties or DataObjects.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ComponentProperty, self).__init__()

        self.citations = []                               # shared.Citation (0.N)
        self.description = None                           # unicode (0.1)
        self.grid = None                                  # unicode (0.1)
        self.intent = None                                # software.ComponentPropertyIntentType (0.1)
        self.is_represented = None                        # bool (1.1)
        self.long_name = None                             # unicode (0.1)
        self.short_name = None                            # unicode (1.1)
        self.standard_names = []                          # unicode (0.N)
        self.sub_properties = []                          # software.ComponentProperty (0.N)
        self.units = None                                 # shared.UnitType (0.1)
        self.values = []                                  # unicode (0.N)


class Composition(object):
    """A concrete class within the cim v1 type system.

    The set of Couplings used by a Component. Couplings can only occur between child components. That is, a composition must belong to an ancestor component of the components whose fields are being connected.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Composition, self).__init__()

        self.couplings = []                               # unicode (0.N)
        self.description = None                           # unicode (0.1)


class Connection(object):
    """A concrete class within the cim v1 type system.

    A Connection represents a link from a source DataSource to a target DataSource.  These can either be ComponentProperties (ie: the values come from an internal component) or DataObjects (ie: the values come from an external file).   It can be associated with another software component (a transformer).  If present, the rate, lag, timeTransformation, and spatialRegridding override that of the parent coupling.  Note that there is the potential for multiple connectionSource & connectionTarget and multiple couplingSources & couplingTargets.  This may lead users to wonder how to match up a connection source (a ComponentProperty) with its coupling source (a SoftwareComponent). Clever logic is not required though; because the sources and targets are listed by reference, they can be found in a CIM document and the parent can be navigated to from there - there is no need to consult the source or target of the coupling.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Connection, self).__init__()

        self.description = None                           # unicode (0.1)
        self.priming = None                               # shared.DataSource (0.1)
        self.properties = []                              # software.ConnectionProperty (0.N)
        self.sources = []                                 # software.ConnectionEndpoint (0.N)
        self.spatial_regridding = []                      # software.SpatialRegridding (0.N)
        self.target = None                                # software.ConnectionEndpoint (0.1)
        self.time_lag = None                              # unicode (0.1)
        self.time_profile = None                          # software.Timing (0.1)
        self.time_transformation = None                   # software.TimeTransformation (0.1)
        self.transformers = []                            # software.ProcessorComponent (0.N)
        self.type = None                                  # software.ConnectionType (0.1)


class ConnectionEndpoint(object):
    """A concrete class within the cim v1 type system.

    The source/target of a coupling.  This is a DataSource (a SoftwareComponent or DataObject).  This is a separate class in order to associate an instanceID with the DataSource; this is used to identify which particular instance is being coupled in case the same DataSource is used more than once in a coupled model (this may be required for BFG).

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ConnectionEndpoint, self).__init__()

        self.data_source = None                           # shared.DataSource (0.1)
        self.instance_id = None                           # unicode (0.1)
        self.properties = []                              # software.ConnectionProperty (0.N)


class ConnectionProperty(shared.Property):
    """A concrete class within the cim v1 type system.

    A ConnectionProperty is a name/value pair used to specify OASIS-specific properties.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ConnectionProperty, self).__init__()



class Coupling(object):
    """A concrete class within the cim v1 type system.

    A coupling represents a set of Connections between a source and target component. Couplings can be complete or incomplete. If they are complete then they must include all Connections between model properties. If they are incomplete then the connections can be underspecified or not listed at all.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Coupling, self).__init__()

        self.connections = []                             # software.Connection (0.N)
        self.description = None                           # unicode (0.1)
        self.is_fully_specified = None                    # bool (1.1)
        self.priming = None                               # shared.DataSource (0.1)
        self.properties = []                              # software.CouplingProperty (0.N)
        self.purpose = None                               # shared.DataPurpose (1.1)
        self.sources = []                                 # software.CouplingEndpoint (1.N)
        self.spatial_regriddings = []                     # software.SpatialRegridding (0.N)
        self.target = None                                # software.CouplingEndpoint (1.1)
        self.time_lag = None                              # software.TimeLag (0.1)
        self.time_profile = None                          # software.Timing (0.1)
        self.time_transformation = None                   # software.TimeTransformation (0.1)
        self.transformers = []                            # software.ProcessorComponent (0.N)
        self.type = None                                  # software.ConnectionType (0.1)


class CouplingEndpoint(object):
    """A concrete class within the cim v1 type system.

    The source/target of a coupling.  This is a DataSource (a SoftwareComponent or DataObject).  This is a separate class in order to associate an instanceID with the DataSource; this is used to identify which particular instance is being coupled in case the same DataSource is used more than once in a coupled model (this may be required for BFG).

    """
    def __init__(self):
        """Instance constructor.

        """
        super(CouplingEndpoint, self).__init__()

        self.data_source = None                           # shared.DataSource (0.1)
        self.instance_id = None                           # unicode (0.1)
        self.properties = []                              # software.CouplingProperty (0.N)


class CouplingProperty(shared.Property):
    """A concrete class within the cim v1 type system.

    A CouplingProperty is a name/value pair used to specify OASIS-specific properties.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(CouplingProperty, self).__init__()



class Deployment(object):
    """A concrete class within the cim v1 type system.

    Gives information about the technical properties of a component: what machine it was run on, which compilers were used, how it was parallised, etc. A deployment basically associates a deploymentDate with a Platform. A deployment only exists if something has been deployed. A platform, in contrast, can exist independently, waiting to be used in deployments.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Deployment, self).__init__()

        self.deployment_date = None                       # datetime.datetime (0.1)
        self.description = None                           # unicode (0.1)
        self.executable_arguments = []                    # unicode (0.N)
        self.executable_name = None                       # unicode (0.1)
        self.parallelisation = None                       # software.Parallelisation (0.1)
        self.platform = None                              # shared.Platform (0.1)


class EntryPoint(object):
    """A concrete class within the cim v1 type system.

    Describes a function or subroutine of a SoftwareComponent. BFG will use these EntryPoints to define a schedule of subroutine calls for a coupled model. Currently, a very basic schedule can be approximated by using the "proceeds" and "follows" attributes, however a more complete system is required for full BFG compatibility. Every EntryPoint can have a set of arguments associated with it. These reference (previously defined) ComponentProperties.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(EntryPoint, self).__init__()

        self.name = None                                  # unicode (0.1)


class Parallelisation(object):
    """A concrete class within the cim v1 type system.

    Describes how a deployment has been parallelised across a computing platform.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Parallelisation, self).__init__()

        self.processes = None                             # int (1.1)
        self.ranks = []                                   # software.Rank (0.N)


class Rank(object):
    """A concrete class within the cim v1 type system.

    Creates and returns instance of rank class.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Rank, self).__init__()

        self.increment = None                             # int (0.1)
        self.max = None                                   # int (0.1)
        self.min = None                                   # int (0.1)
        self.value = None                                 # int (0.1)


class SpatialRegridding(object):
    """A concrete class within the cim v1 type system.

    Characteristics of the scheme used to interpolate a field from one grid (source grid) to another (target grid).  Documents should use either the spatialRegriddingStandardMethod _or_ the spatialRegriddingUserMethod, but not both.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(SpatialRegridding, self).__init__()

        self.dimension = None                             # software.SpatialRegriddingDimensionType (0.1)
        self.properties = []                              # software.SpatialRegriddingProperty (0.N)
        self.standard_method = None                       # software.SpatialRegriddingStandardMethodType (0.1)
        self.user_method = None                           # software.SpatialRegriddingUserMethod (0.1)


class SpatialRegriddingProperty(shared.Property):
    """A concrete class within the cim v1 type system.

    Used for OASIS-specific regridding information (ie: masked, order, normalisation, etc.)

    """
    def __init__(self):
        """Instance constructor.

        """
        super(SpatialRegriddingProperty, self).__init__()



class SpatialRegriddingUserMethod(object):
    """A concrete class within the cim v1 type system.

    Characteristics of the scheme used to interpolate a field from one grid (source grid) to another (target grid).  Documents should use either the spatialRegriddingStandardMethod _or_ the spatialRegriddingUserMethod, but not both.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(SpatialRegriddingUserMethod, self).__init__()

        self.file = None                                  # data.DataObject (0.1)
        self.name = None                                  # unicode (1.1)


class TimeLag(object):
    """A concrete class within the cim v1 type system.

    The coupling field used in the target at a given time corresponds to a field produced by the source at a previous time. This lag specifies the difference in time.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(TimeLag, self).__init__()

        self.units = None                                 # software.TimingUnits (0.1)
        self.value = None                                 # int (0.1)


class TimeTransformation(object):
    """A concrete class within the cim v1 type system.

    The coupling field used in the target at a given time corresponds to a field produced by the source at a previous time. This lag specifies the difference in time.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(TimeTransformation, self).__init__()

        self.description = None                           # unicode (0.1)
        self.mapping_type = None                          # software.TimeMappingType (1.1)


class Timing(object):
    """A concrete class within the cim v1 type system.

    Provides information about the rate of couplings and connections and/or the timing characteristics of individual components - for example, the start and stop times that the component was run for or the units of time that a component is able to model (in a single timestep).

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Timing, self).__init__()

        self.end = None                                   # datetime.datetime (0.1)
        self.is_variable_rate = None                      # bool (0.1)
        self.rate = None                                  # int (0.1)
        self.start = None                                 # datetime.datetime (0.1)
        self.units = None                                 # software.TimingUnits (0.1)


class ModelComponent(Component):
    """A concrete class within the cim v1 type system.

    A ModelComponent is a scientific model; it represents code which models some physical phenomena for a particular length of time.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ModelComponent, self).__init__()

        self.activity = None                              # activity.Activity (0.1)
        self.meta = shared.DocMetaInfo()                  # shared.DocMetaInfo (1.1)
        self.timing = None                                # software.Timing (0.1)
        self.type = None                                  # software.ModelComponentType (0.1)
        self.types = []                                   # software.ModelComponentType (1.N)


class ProcessorComponent(Component):
    """A concrete class within the cim v1 type system.

    A ModelComponent is a scientific model; it represents code which models some physical phenomena for a particular length of time.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ProcessorComponent, self).__init__()

        self.meta = shared.DocMetaInfo()                  # shared.DocMetaInfo (1.1)


class StatisticalModelComponent(Component):
    """A concrete class within the cim v1 type system.

    Creates and returns instance of statistical_model_component class.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(StatisticalModelComponent, self).__init__()

        self.meta = shared.DocMetaInfo()                  # shared.DocMetaInfo (1.1)
        self.timing = None                                # software.Timing (0.1)
        self.type = None                                  # software.StatisticalModelComponentType (0.1)
        self.types = []                                   # software.StatisticalModelComponentType (1.N)


class ComponentPropertyIntentType(object):
    """An enumeration within the cim v1 type system.

    The direction that the associated component property is intended to be coupled: in, out, or inout.
    """
    is_open = False
    members = [
        "in",
        "out",
        "inout"
        ]
    descriptions = [
        "None",
        "None",
        "None"
        ]


class ConnectionType(object):
    """An enumeration within the cim v1 type system.

    The ConnectionType enumeration describes the mechanism of transport for a connection.
    """
    is_open = True
    members = []
    descriptions = []


class CouplingFrameworkType(object):
    """An enumeration within the cim v1 type system.

    Creates and returns instance of coupling_framework_type enum.
    """
    is_open = False
    members = [
        "BFG",
        "ESMF",
        "OASIS"
        ]
    descriptions = [
        "None",
        "None",
        "None"
        ]


class ModelComponentType(object):
    """An enumeration within the cim v1 type system.

    An enumeration of types of ModelComponent. This includes things like atmosphere & ocean models, radiation schemes, etc. CIM best-practice is to describe every component for which there is a named ComponentType as a separate component, even if it is not a separate unit of software (ie: even if it is embedded), instead of as a (set of) ModelParameters. This codelist is synonomous with "realm" for the purposes of CMIP5.
    """
    is_open = True
    members = []
    descriptions = []


class SpatialRegriddingDimensionType(object):
    """An enumeration within the cim v1 type system.

    Creates and returns instance of spatial_regridding_dimension_type enum.
    """
    is_open = False
    members = [
        "1D",
        "2D",
        "3D"
        ]
    descriptions = [
        "None",
        "None",
        "None"
        ]


class SpatialRegriddingStandardMethodType(object):
    """An enumeration within the cim v1 type system.

    Creates and returns instance of spatial_regridding_standard_method_type enum.
    """
    is_open = False
    members = [
        "linear",
        "near-neighbour",
        "cubic",
        "conservative-first-order",
        "conservative-second-order",
        "conservative",
        "non-conservative"
        ]
    descriptions = [
        "None",
        "None",
        "None",
        "None",
        "None",
        "None",
        "None"
        ]


class StatisticalModelComponentType(object):
    """An enumeration within the cim v1 type system.

    An enumeration of types of ProcessorComponent.  This includes things like transformers and post-processors.
    """
    is_open = True
    members = []
    descriptions = []


class TimeMappingType(object):
    """An enumeration within the cim v1 type system.

    Enumerates the different ways that time can be mapped when transforming from one field to another.
    """
    is_open = True
    members = []
    descriptions = []


class TimingUnits(object):
    """An enumeration within the cim v1 type system.

    Creates and returns instance of timing_units enum.
    """
    is_open = False
    members = [
        "seconds",
        "minutes",
        "hours",
        "days",
        "months",
        "years",
        "decades",
        "centuries"
        ]
    descriptions = [
        "None",
        "None",
        "None",
        "None",
        "None",
        "None",
        "None",
        "None"
        ]





import pyesdoc

print pyesdoc.search("cmip5", "models")
print pyesdoc.search("cmip5", "models", document_version="*")
print pyesdoc.search("cmip5", "models", institute="ipsl")
print pyesdoc.search("cmip6", "experiments")
print pyesdoc.search("cmip6", "experiments", sub_project="damip")
print pyesdoc.search("cmip6", "simulations")
print pyesdoc.search("cmip6", "machines")
print pyesdoc.search("cmip6", "performances")



UID = "ec9243df-26b7-4ed5-ba77-beb65bc13276"

d = pyesdoc.retrieve(UID)

d.name += "-666"
pyesdoc.publish(d)
d = pyesdoc.retrieve(UID)
assert d.name.endswith("-666")

pyesdoc.unpublish(d.meta.id, d.meta.version)
d = pyesdoc.retrieve(UID, d.meta.version)
assert d is None



# -*- coding: utf-8 -*-

"""
.. module:: test_type_cim_v1_activity_simulation_run.py

   :license: GPL / CeCILL
   :platform: Unix, Windows
   :synopsis: Tests a cim.v1.SimulationRun instance.

.. moduleauthor:: Earth System Documentation (ES-DOC) <dev@es-doc.org>

"""
import arrow

import pyesdoc.ontologies.cim as cim
import test_utils as tu



# Test type.
DOC_TYPE = cim.v1.SimulationRun

# Test type display name.
DOC_TYPE_DISPLAY_NAME = "Simulation"

# Test document type.
DOC_TYPE_KEY = DOC_TYPE.type_key

# Test representation file.
DOC_FILE = 'cim.1.activity.SimulationRun.xml-metafor-cim-v1'

# Test document uid.
DOC_UID = 'e4155a28-268e-11e1-9a0e-00163e9152a5'

# Test document version.
DOC_VERSION = '2'

# Test document creation date.
DOC_DATE = arrow.get('2012-04-23 14:59:06.767729').datetime

# Test document project.
DOC_PROJECT = "CMIP5"

# Test document project.
DOC_INSTITUTE = "MOHC"

# Test document author.
DOC_AUTHOR = "Metafor Questionnaire"


def assert_meta_info(meta):
    """Asserts a document's meta-information.

    :param object meta: Document meta information.

    """
    tu.assert_iter(meta.external_ids, 2, cim.v1.StandardName)
    id = meta.external_ids[0]
    tu.assert_bool(id.is_open, True)
    tu.assert_iter(id.standards, 1, cim.v1.Standard)
    tu.assert_str(id.value, "MOHC_HadGEM2-A_amip_r1i1p1_1978")
    std = id.standards[0]
    tu.assert_str(std.description, "The QN_DRS value allows", True)
    tu.assert_str(std.name, "QN_DRS")


def assert_extension_info(ext):
    """Asserts a document's extension information.

    :param object ext: Document extension information.

    """
    tu.assert_str(ext.display_name, "amip")
    tu.assert_str(ext.description, "Main amip ensemble simulations", True)
    tu.assert_str(ext.full_display_name, "CMIP5 Simulation : MOHC - amip")
    tu.assert_int(ext.summary_fields, 2)
    tu.assert_str(ext.summary_fields[0], "amip")
    tu.assert_str(ext.summary_fields[1], "3.3 AMIP and AMIP Ensemble", True)


def _assert_doc_core(doc, is_update):
    """Assert core information."""
    tu.assert_str(doc.authors, "C. Jones, J. Hughes", True)
    tu.assert_object(doc.date_range, cim.v1.ClosedDateRange)
    tu.assert_str(doc.date_range.duration, "P30Y")
    tu.assert_date(doc.date_range.start, "1978-12-01 00:00:00+00:00")
    tu.assert_str(doc.description, "Main amip ensemble simulations", True)
    tu.assert_str(doc.long_name, "3.3 AMIP and AMIP Ensemble", True)
    tu.assert_iter(doc.projects, 1, str)
    tu.assert_str(doc.projects[0], "CMIP5")
    tu.assert_iter(doc.responsible_parties, 4, cim.v1.ResponsibleParty)
    tu.assert_str(doc.short_name, "amip")
    tu.assert_object(doc.spinup_date_range, cim.v1.ClosedDateRange)
    tu.assert_str(doc.spinup_date_range.duration, "P30Y")
    tu.assert_date(doc.spinup_date_range.start, "1978-12-01 00:00:00+00:00")


def _assert_doc_conformances(doc, is_update):
    """Assert simulation conformances."""
    tu.assert_iter(doc.conformances, 13, cim.v1.Conformance)

    c = doc.conformances[0]
    tu.assert_str(c.type, "standard config")
    tu.assert_iter(c.requirements_references, 1, cim.v1.DocReference)
    ref = c.requirements_references[0]
    tu.assert_str(ref.description, "Reference to a NumericalRequirement", True)
    tu.assert_uuid(ref.id, "9fa513fc-d3a5-11df-837f-00163e9152a5")
    tu.assert_str(ref.name, "ic.003")
    tu.assert_str(ref.type, "NumericalRequirement")
    tu.assert_int(ref.version, 1)

    c = doc.conformances[7]
    tu.assert_str(c.description, "Prescribed land use", True)
    tu.assert_bool(c.is_conformant, True)
    tu.assert_str(c.type, "via inputs")


def _assert_doc_deployments(doc, is_update):
    """Assert simulation deployments."""
    tu.assert_iter(doc.deployments, 1, cim.v1.Deployment)

    dep = doc.deployments[0]
    tu.assert_str(dep.description, "The resources(deployment) on which", True)
    tu.assert_object(dep.platform_reference, cim.v1.DocReference)
    ref = dep.platform_reference
    tu.assert_str(ref.description, "Reference to a platform", True)
    tu.assert_uuid(ref.id, "b765775a-e2ac-11df-9efb-00163e9152a5")
    tu.assert_str(ref.name, "IBM Power 6")
    tu.assert_str(ref.type, "platform")
    tu.assert_int(ref.version, 1)


def _assert_doc_inputs(doc, is_update):
    """Assert simulation inputs."""
    tu.assert_iter(doc.inputs, 56, cim.v1.Coupling)


def _assert_doc_references(doc, is_update):
    """Assert simulation references."""
    # Model reference.
    tu.assert_object(doc.model_reference, cim.v1.DocReference)
    ref = doc.model_reference
    tu.assert_str(ref.description, "Reference to a modelComponent", True)
    tu.assert_uuid(ref.id, "7a2b64cc-03ca-11e1-a36a-00163e9152a5")
    tu.assert_str(ref.name, "HadGEM2-A")
    tu.assert_str(ref.type, "modelComponent")
    tu.assert_int(ref.version, 1)

    # Experiment reference.
    tu.assert_iter(doc.supports_references, 1, cim.v1.DocReference)
    ref = doc.supports_references[0]
    tu.assert_str(ref.description, "Reference to an Experiment", True)
    tu.assert_uuid(ref.id, "9fa513fc-d3a5-11df-837f-00163e9152a5")
    tu.assert_str(ref.name, "amip")
    tu.assert_str(ref.type, "experiment")
    tu.assert_int(ref.version, 1)


def assert_doc(doc, is_update=False):
    """Asserts a document.

    :param object doc: Document being tested.
    :param bool is_update: Flag indicating whether document has been updated.

    """
    for assertor in (
        _assert_doc_core,
        _assert_doc_conformances,
        _assert_doc_deployments,
        _assert_doc_inputs,
        _assert_doc_references,
        ):
        assertor(doc, is_update)


def assert_doc1(doc, meta, ext):
    """Asserts a document.

    :param object doc: Document being tested.
    :param object meta: Document meta information.
    :param object ext: Document extension information.

    """
    assert len(doc.meta.external_ids) == 1
    assert doc.meta.external_ids[0].is_open == True
    assert doc.meta.external_ids[0].value == 'MOHC_HadGEM2-ES_6.3 abrupt4xCO2'
    assert len(doc.meta.external_ids[0].standards) == 1
    assert doc.meta.external_ids[0].standards[0].name == 'QN_DRS'
    assert doc.meta.external_ids[0].standards[0].description.startswith('The QN_DRS value')

    assert doc.long_name == '6.3  Gregory-style diagnosis of slow climate system responses'
    assert doc.short_name == 'abrupt4xCO2'
    assert doc.description.startswith('Imposes an instantaneous')

    assert doc.authors == 'M Webb, J. Williams, T. Andrews, A. Bodas-Salcedo, Y Tsushima, J Hughes, C Jones'

    assert doc.calendar is not None
    assert isinstance(doc.calendar, cim.v1.PerpetualPeriod) == True

    assert len(doc.conformances) == 3
    assert doc.conformances[0].is_conformant == True
    assert doc.conformances[0].type == 'via inputs'
    assert doc.conformances[0].description.startswith('Main element (r1i1p1)')
    assert doc.conformances[0].frequency in [None, '']
    assert len(doc.conformances[0].requirements_references) == 1
    assert doc.conformances[0].requirements_references[0].id == uuid.UUID('a0d3a554-d3a5-11df-837f-00163e9152a5')
    assert doc.conformances[0].requirements_references[0].name == 'ic.006'
    assert doc.conformances[0].requirements_references[0].type == 'NumericalRequirement'
    assert doc.conformances[0].requirements_references[0].version == '1'
    assert doc.conformances[0].requirements_references[0].description == 'Reference to a NumericalRequirement called ic.006 in a experiment called 6.3 abrupt4xCO2'

    assert doc.control_simulation is None
    assert doc.control_simulation_reference is not None
    assert doc.control_simulation_reference.id == uuid.UUID('354ed4ec-e346-11df-9343-00163e9152a5')
    assert doc.control_simulation_reference.name == 'piControl'
    assert doc.control_simulation_reference.type == 'simulation'
    assert doc.control_simulation_reference.version == '4'
    assert doc.control_simulation_reference.description == 'branch date 1859-12-01'

    assert len(doc.deployments) == 1
    assert doc.deployments[0].description.startswith('The resources(deployment) on which this simulation ran')
    assert doc.deployments[0].platform == None
    assert doc.deployments[0].platform_reference is not None
    assert doc.deployments[0].platform_reference.id == uuid.UUID('b765775a-e2ac-11df-9efb-00163e9152a5')
    assert doc.deployments[0].platform_reference.name == 'IBM Power 6'
    assert doc.deployments[0].platform_reference.type == 'platform'
    assert doc.deployments[0].platform_reference.version == '1'
    assert doc.deployments[0].platform_reference.description == 'Reference to a platform called IBM Power 6'

    assert isinstance(doc.date_range, cim.v1.ClosedDateRange)
    assert doc.date_range.duration == 'P150Y'
    assert doc.date_range.start == arrow.get('1859-12-01 00:00:00Z').datetime

    assert len(doc.inputs) == 34

    assert len(doc.inputs[0].connections) == 1
    assert len(doc.inputs[0].connections[0].sources) == 1
    assert doc.inputs[0].connections[0].sources[0].data_source_reference is not None
    assert doc.inputs[0].connections[0].sources[0].data_source_reference.id == uuid.UUID('a31d55c4-4bc0-11e0-a872-00163e9152a5')
    assert doc.inputs[0].connections[0].target.data_source_reference is not None
    assert doc.inputs[0].connections[0].target.data_source_reference.id == uuid.UUID('317f51b8-e2b3-11df-bf17-00163e9152a5')

    assert len(doc.inputs[0].sources) == 1
    assert doc.inputs[0].sources[0].data_source_reference is not None
    assert doc.inputs[0].sources[0].data_source_reference.id == uuid.UUID('a31d55c4-4bc0-11e0-a872-00163e9152a5')
    assert doc.inputs[0].sources[0].data_source_reference.name == 'well_mixed_gas_CO2'
    assert doc.inputs[0].sources[0].data_source_reference.type == 'dataObject'
    assert doc.inputs[0].sources[0].data_source_reference.version == '1'
    assert doc.inputs[0].sources[0].data_source_reference.description == 'Reference to a dataObject called well_mixed_gas_CO2'

    assert doc.inputs[0].target.data_source_reference is not None
    assert doc.inputs[0].target.data_source_reference.id == uuid.UUID('317f51b8-e2b3-11df-bf17-00163e9152a5')
    assert doc.inputs[0].target.data_source_reference.name == 'Atmosphere'
    assert doc.inputs[0].target.data_source_reference.type == 'modelComponent'
    assert doc.inputs[0].target.data_source_reference.version == '1'
    assert doc.inputs[0].target.data_source_reference.description == 'Reference to a modelComponent called Atmosphere'

    assert doc.inputs[0].description.startswith('An instantaneous quadrupling')
    assert doc.inputs[0].is_fully_specified == True
    assert doc.inputs[0].purpose == 'boundaryCondition'
    assert doc.inputs[0].type == 'File'

    assert doc.inputs[0].time_profile.is_variable_rate == False
    assert doc.inputs[0].time_profile.rate == 30
    assert doc.inputs[0].time_profile.units == 'minutes'

    assert doc.inputs[0].time_transformation is not None
    assert doc.inputs[0].time_transformation.mapping_type == 'TimeInterpolation'

    assert len(doc.inputs[33].spatial_regriddings) == 1
    assert doc.inputs[33].spatial_regriddings[0].standard_method == 'conservative'

    assert doc.model == None
    assert doc.model_reference is not None
    assert doc.model_reference.id == uuid.UUID('309f6a26-e2b3-11df-bf17-00163e9152a5')
    assert doc.model_reference.name == 'HadGEM2-ES'
    assert doc.model_reference.type == 'modelComponent'
    assert doc.model_reference.version == '2'
    assert doc.model_reference.description == 'Reference to a modelComponent called HadGEM2-ES'

    assert len(doc.projects) == 1
    assert doc.projects[0] == 'CMIP5'

    assert len(doc.responsible_parties) == 4
    rp = doc.responsible_parties[0]
    assert rp.abbreviation == 'Mark Webb'
    assert rp.email == 'mark.webb@metoffice.gov.uk'
    assert rp.individual_name == 'Mark Webb'
    assert rp.role == 'contact'

    assert len(doc.supports_references) == 1
    assert doc.supports_references[0].id == uuid.UUID('a0d3a554-d3a5-11df-837f-00163e9152a5')
    assert doc.supports_references[0].name == 'abrupt4xCO2'
    assert doc.supports_references[0].type == 'experiment'
    assert doc.supports_references[0].version == '1'
    assert doc.supports_references[0].description == 'Reference to an Experiment called abrupt4xCO2 with experimentNumber 6.3'


def update_doc(doc):
    """Update a document prior to republishing.

    :param object doc: Document being republished.

    """
    pass


def assert_doc_updates(doc):
    """Asserts a document after being updated.

    :param object doc: Document being tested.

    """
    pass


"""

A realm process sepecialization.

For further information goto http://wordpress.es-doc.org/cmip6-model-specializations.
"""
# --------------------------------------------------------------------
# INTERNAL (do not change)
# --------------------------------------------------------------------
from collections import OrderedDict

DETAILS = OrderedDict()
ENUMERATIONS = OrderedDict()

# --------------------------------------------------------------------
# CONTACT: Set to realm specialization co-ordinator.
# --------------------------------------------------------------------
CONTACT = 'Charlotte Pascoe'

# --------------------------------------------------------------------
# AUTHORS: Set to realm specialization authors (comma delimited).
# --------------------------------------------------------------------
AUTHORS = 'Charlotte Pascoe, Robert Pincus (NOAA)'

# --------------------------------------------------------------------
# QUALITY CONTROL STATUS: Set to 'draft' or 'complete'
# --------------------------------------------------------------------
QC_STATUS = 'draft'

# --------------------------------------------------------------------
# DESCRIPTION: Short description of the specialization.
# --------------------------------------------------------------------
DESCRIPTION = 'Characteristics of the cloud scheme'

# --------------------------------------------------------------------
# PROCESS: top level properties
# --------------------------------------------------------------------
DETAILS['toplevel'] = {
    'description': "Top level cloud scheme process properties",
    'properties': [
        ('name', 'str', '0.1',
            'Commonly used name for the cloud scheme'),
        ('atmos_coupling', 'ENUM:atmos_cloud_coupling', '0.N',
            'Atmosphere components that are linked to the cloud scheme'),
        ('uses_separate_treatment', 'bool', '1.1',
            'Different cloud schemes for the different types of clouds (convective, stratiform and boundary layer)'),
        ('cloud_overlap_method', 'ENUM:cloud_overlap_method', '1.1',
            'Method for taking into account overlapping of cloud layers'),
        ('cloud_inhomogeneity', 'ENUM:inhomogeneity_treatment', '1.1',
            'Method for taking into account horizontal cloud inhomogeneity'),
        ('processes', 'ENUM:processes_attributes', '1.N',
            'Processes included in the cloud scheme'),
        ]
    }

# --------------------------------------------------------------------
# SUB-PROCESS: sub_grid_scale_water_distribution
# --------------------------------------------------------------------
DETAILS['sub_grid_scale_water_distribution'] = {
    'description': 'Sub-grid scale water distribution',
    'properties': [
        ('type', 'ENUM:sub_grid_scale_water_distribution_type', '1.1',
            'Sub-grid scale water distribution type'),
        ('function_name', 'str', '1.1',
            'Sub-grid scale water distribution function name'),
        ('function_order', 'int', '1.1',
            'Sub-grid scale water distribution function type'),
        ('convection_coupling', 'ENUM:sub_grid_scale_water_distribution_convection', '1.N',
            'Sub-grid scale water distribution coupling with convection'),
        ]
    }

# --------------------------------------------------------------------
# PROCESS: ENUMERATIONS
# --------------------------------------------------------------------

ENUMERATIONS['sub_grid_scale_water_distribution_type'] = {
    'description': 'Approach used for cloud water content and fractional cloud cover',
    'is_open': False,
    'members': [
        ('prognostic', None),
        ('diagnostic', None),
        ]
    }

ENUMERATIONS['sub_grid_scale_water_distribution_convection'] = {
    'description': 'Type(s) of convection that the formation of clouds is coupled with',
    'is_open': False,
    'members': [
        ('coupled with deep', None),
        ('coupled with shallow', None),
        ('not coupled with convection', None),
        ]
    }

ENUMERATIONS['atmos_cloud_coupling'] = {
    'description': 'Atmosphere components that are linked to the cloud scheme',
    'is_open': False,
    'members': [
        ('atmosphere_radiation', None),
        ('atmosphere_microphysics_precipitation', None),
        ('atmosphere_turbulence_convection', None),
        ('atmosphere_gravity_waves', None),
        ('atmosphere_solar', None),
        ('atmosphere_volcano', None),
        ('atmosphere_cloud_simulator', None)
    ]
}

ENUMERATIONS['cloud_overlap_method'] = {
    'description': 'Cloud scheme cloud overlap method',
    'is_open': True,
    'members': [
        ('random', None),
        ('maximum', None),
        ('maximum-random', 'combination of maximum and random overlap between clouds'),
        ('exponential', None),
        ]
    }

ENUMERATIONS['inhomogeneity_treatment'] = {
    'description': 'Cloud scheme inhomogeneity treatment',
    'is_open': True,
    'members': [
        ('Monte Carlo Independent Column Approximation', 'McICA'),
        ('Triplecloud', 'Regions of clear sky, optically thin cloud and optically thick cloud, Shonk et al 2010'),
        ('analytic', None),
        ]
    }
ENUMERATIONS['processes_attributes'] = {
    'description': 'Processes included in the cloud scheme.',
    'is_open': True,
    'members': [
        ('entrainment', None),
        ('detrainment', None),
        ('bulk cloud', None),
        ]
    }


"""
.. module:: pyesdoc.mp.ontologies.generators.__init__.py
   :license: GPL/CeCIL
   :platform: Unix, Windows
   :synopsis: pyesdoc.mp generators sub-package initializer.

.. moduleauthor:: Mark Conway-Greenslade <momipsl@ipsl.jussieu.fr>


"""
import os

from pyesdoc.mp import utils
from pyesdoc.mp.ontologies.core.factory import create_ontology
from pyesdoc.mp.ontologies.core.schema_validation import validate as validate_schema
from pyesdoc.mp.ontologies.generators import generator_utils as gu
from pyesdoc.mp.ontologies.generators import python
from pyesdoc.mp.ontologies.generators import qxml
from pyesdoc.mp.ontologies.generators import qconfig
from pyesdoc.mp.ontologies.generators.generator_context import GeneratorContext


# Map of generator handlers.
_HANDLERS = { i.__name__.split('.')[-1]: i for i in (python, qxml, qconfig) }


def _log_start(schema, language, io_dir):
    """Informs user that generation is about to begin.

    """
    utils.log("Welcome to the ES-DOC meta-programming code generator !")
    utils.log("GENERATION OPTION : ontology schema = {0} v{1}".format(schema.NAME, schema.VERSION))
    utils.log("GENERATION OPTION : programming language = {0}".format(language))
    utils.log("GENERATION OPTION : output directory = {0}".format(io_dir))


def _log_end():
    """Informs user that generation is complete.

    """
    utils.log("Thank you for using the ES-DOC code generator")


def _can_generate(schema, language, output_dir):
    """Verifies whether the generation options are in a state such that generation can occur.

    :param module schema: Ontology schema definition.
    :param str language: Target programming language.
    :param str io_dir: Target I/O directory.

    :returns: True if generation can occur, False otherwise.
    :rtype: bool

    """
    if not language in _HANDLERS:
        err = "Programming language is unsupported [{}].  Supported languages are {}."
        err = err.format(language, _HANDLERS.keys())
        raise ValueError(err)
    if not os.path.exists(output_dir):
        raise IOError("Output directory does not exist [{0}].".format(output_dir))

    errors = validate_schema(schema)
    if errors:
        utils.log("-------------------------------------------------------------------")
        for error in errors:
            utils.log("VALIDATION ERROR :: {0}".format(error))
        return False

    return True


def _generate_from_template(ctx, template):
    """Generates code from a tornado template.

    """
    lu = _HANDLERS[ctx.language].UTILS
    utils.log("GENERATOR = {0} :: generation begins".format(ctx.key))
    code = ctx.get_code(template, lu)
    if code:
        gu.write_file(gu.format_code(ctx, code),
                      lu.get_ontology_directory(ctx),
                      lu.get_module_file_name(template.split('.')[0]))
    utils.log("GENERATOR = {0} :: generation complete".format(ctx.key))


def _generate_from_parser(ctx, parser_type):
    """Generates code from an ontology parser.

    """
    parser = parser_type()
    if not parser.is_required(ctx):
        utils.log("GENERATOR = {0} :: generation skipped".format(ctx.key))
    else:
        utils.log("GENERATOR = {0} :: generation begins".format(ctx.key))
        parser.execute(ctx)
        for code, dir_, fpath in ctx.code:
            gu.write_file(gu.format_code(ctx, code), dir_, fpath)
        utils.log("GENERATOR = {0} :: generation complete".format(ctx.key))


def generate(schema, language, io_dir):
    """Generates code.

    :param module schema: Ontology schema definition.
    :param str language: Target programming language.
    :param str io_dir: Target I/O directory.

    """
    if not _can_generate(schema, language, io_dir):
        return

    _log_start(schema, language, io_dir)

    # Initialise ontology.
    ontology = create_ontology(schema)
    utils.log("ONTOLOGY :: {0} (packages={1}, classes={2}, enums={3})".format(
        ontology, len(ontology.packages), len(ontology.classes), len(ontology.enums)))

    # Apply language specific pre-generator formatter.
    try:
        formatter = _HANDLERS[language].UTILS.format
    except AttributeError:
        pass
    else:
        formatter(ontology)
        utils.log("ONTOLOGY :: formatted for {0}".format(language))

    # Invoke language specific generators.
    for generator in _HANDLERS[language].GENERATORS:
        func = _generate_from_template if isinstance(generator, str) else _generate_from_parser
        func(GeneratorContext(generator, ontology, language, io_dir), generator)

    _log_end()




"""A realm process sepecialization.

For further information goto http://wordpress.es-doc.org/cmip6-model-specializations.

"""
# --------------------------------------------------------------------
# INTERNAL (do not change)
# --------------------------------------------------------------------
from collections import OrderedDict

DETAILS = OrderedDict()
ENUMERATIONS = OrderedDict()

# --------------------------------------------------------------------
# CONTACT: Set to realm specialization co-ordinator.
# --------------------------------------------------------------------
CONTACT = 'David Hassell'

# --------------------------------------------------------------------
# AUTHORS: Set to realm specialization authors (comma delimited).
# --------------------------------------------------------------------
AUTHORS = 'David Hassell'

# --------------------------------------------------------------------
# QUALITY CONTROL STATUS: Set to 'draft' or 'complete'
# --------------------------------------------------------------------
QC_STATUS = 'draft'

# --------------------------------------------------------------------
# DESCRIPTION: Short description of the specialization.
# --------------------------------------------------------------------
DESCRIPTION = 'Land surface albedo'

# --------------------------------------------------------------------
# ALBEDO: top level
# --------------------------------------------------------------------
DETAILS['toplevel'] = {
    'description': 'General properties in land surface albedo',
    'properties': [
        ('tiling', 'str', '0.1',
             'Describe the albedo tiling, if any.'),
    ],
}

# --------------------------------------------------------------------
# ALBEDO: ENUMERATIONS
# --------------------------------------------------------------------

ENUMERATIONS['phenology_types'] = {
    'description': 'Description of the treatment of vegetation phenolo]gy',
    'is_open': True,
    'members': [
        ('prognostic', None),
        ('diagnostic (vegetation map)', None),
        ]
}

ENUMERATIONS['leaf_area_index_types'] = {
    'description': 'Describe the treatment of the leaf area index',
    'is_open': True,
    'members': [
        ('prescribed', None),
        ('prognostic', None),
        ('diagnostic', None),
        ]
    }

ENUMERATIONS['biomass_types'] = {
    'description': 'Describe the treatment of vegetation biomass',
    'is_open': True,
    'members': [
        ('prognostic', None),
        ('diagnostic', None),
        ]
    }

ENUMERATIONS['biogeography_types'] = {
    'description': 'Describe the treatment of vegatation biogeography',
    'is_open': True,
    'members': [
        ('prognostic', None),
        ('diagnostic', None),
        ]
    }

ENUMERATIONS['stomatal_resistance_types'] = {
    'description': 'Specify the dependancies on vegetation stomatal resistance',
    'is_open': True,
    'members': [
        ('light', None),
        ('temperature', None),
        ('water availability', None),
        ('CO2', None),
        ('O3', None),
    ]
}



# -*- coding: utf-8 -*-

"""
.. module:: science_classes.py
   :synopsis: Set of CIM v2 ontology type definitions.

"""
def model():
    """A model component: can be executed standalone, and has as
    scientific description available via a link to a science.domain
    document. (A configured model can be understood in terms of a
    simulation, a model, and a configuration.).

    """
    return {
        'type': 'class',
        # TODO: review whether science.topic works better.
        'base': 'software.component_base',
        'is_abstract': False,
        'is_document': True,
        'properties': [
            ('model_type', 'science.model_types', '1.1',
                "Generic type for this model."),
            ('key_properties', 'science.topic', '0.1',
                "Model default key properties (may be over-ridden in coupled component and realm properties)."),
            ('activity_properties', 'science.topic', '0.1',
                "Properties specific to the modelling activity in question, e.g. radiative forcing agents for CMIP6."),
            ('realms', 'linked_to(science.realm)', '0.N',
                "The scientific realms which this model simulates internally, i.e. those which are not linked together using a coupler."),

            # TODO: review these attributes once software package is peer-reviewed
            ('coupled_components', 'linked_to(science.model)', '0.N',
                "Software components which are linked together using a coupler to deliver this model."),
            ('coupler', 'software.coupling_framework', '0.1',
                "Overarching coupling framework for model."),
            ('internal_software_components', 'software.software_component', '0.N',
                "Software modules which together provide the functionality for this model."),
        ]
    }


def realm():
    """Scientific area of a numerical model - usually a sub-model or component.

    """
    return {
        'type': 'class',
        'base': 'science.topic',
        'is_abstract': False,
        'is_document': True,
        'pstr': ('{}', ('canonical_name')),        
        'properties': [
            ('canonical_name', 'str', '0.1',
                "Canonical name for the realm."),
            ('grid', 'science.topic', '0.1',
                "The grid used to layout the variables (e.g. the Global ENDGAME-grid)."),
            ('key_properties', 'science.topic', '0.1',
                "Realm key properties which differ from model defaults (grid, timestep etc)."),
            ('model', 'science.model', '1.1',
                "Associated model."),
            ('processes', 'science.topic', '1.N',
                "Processes simulated within the realm."),

            # TODO: review these attributes once software package is peer-reviewed
            ('software_frameworks', 'software.implementation', '0.N',
                "Software framework(s) of the realm."),
        ]
    }


def topic():
    """An organized collection of details upon a specific topic, e.g. model key properties.

    """
    return {
        'type': 'class',
        'base': None,
        'is_abstract': False,
        'pstr': ('{}', ('short_name')),        
        'properties': [
            ('citations', 'shared.citation', '0.N',
                "Set of pertinent citations."),
            ('description', 'str', '1.1',
                "A description (derived from specialization)."),
            ('keywords', 'str', '0.N',
                "Keywords to help re-use and discovery of this information."),
            ('overview', 'str', '0.1',
                "An overview of topic being described."),
            ('properties', 'science.topic_property', '0.N',
                "Set of associated specialized properties."),
            ('property_sets', 'science.topic_property_set', '0.N',
                "Grouped specialized properties."),
            ('responsible_parties', 'shared.responsibility', '0.N',
                "People or organisations responsible for providing this information."),
            ('short_name', 'str', '1.1',
                "A short-name / key (derived from specialization)."),
            ('specialization_id', 'str', '1.1',
                "Specialization identifier (derived from specialization)."),
            ('sub_topics', 'science.topic', '0.N',
                "Discrete sub-topic with details.")
        ]
    }


def topic_property_set():
    """Provides specific details related to a topic (i.e. process, sub-process,
    grid, key properties, etc).

    """
    return {
        'type': 'class',
        'base': None,
        'is_abstract': False,
        'pstr': ('{}', ('specialization_id',)),        
        'properties': [
            ('description', 'str', '1.1',
                "A description (derived from specialization)."),
            ('properties', 'science.topic_property', '1.N',
                "Set of associated specialized properties."),
            ('short_name', 'str', '1.1',
                "A short-name / key (derived from specialization)."),
            ('specialization_id', 'str', '1.1',
                "Specialization identifier (derived from specialization)."),
        ]
    }


def topic_property():
    """A specialized question asked of the scientic community.

    """
    return {
        'type': 'class',
        'base': None,
        'is_abstract': False,
        'pstr': ('{}: {}', ('specialization_id', 'values',)),        
        'properties': [
            ('qc_status', 'int', '1.1',
                "Quality control status of entered values."),
            ('specialization_id', 'str', '1.1',
                "Specialization identifier (derived from specialization)."),
            ('values', 'str', '1.N',
                "User value(s)."),
        ]
    }



"""A realm key-properties specialization.

For further information goto http://wordpress.es-doc.org/cmip6-model-specializations.

"""
# --------------------------------------------------------------------
# INTERNAL (do not change)
# --------------------------------------------------------------------
from collections import OrderedDict

DETAILS = OrderedDict()
ENUMERATIONS = OrderedDict()

# --------------------------------------------------------------------
# CONTACT: Set to specialization co-ordinator.
# --------------------------------------------------------------------
CONTACT = 'Charlotte Pascoe, David Hassell'

# --------------------------------------------------------------------
# AUTHORS: Set to specialization authors (comma delimited).
# --------------------------------------------------------------------
AUTHORS = 'David Hassell'

# --------------------------------------------------------------------
# QUALITY CONTROL STATUS: Set to 'draft' or 'complete'
# --------------------------------------------------------------------
QC_STATUS = 'draft'

# --------------------------------------------------------------------
# DESCRIPTION: Short description of the specialization.
# --------------------------------------------------------------------
DESCRIPTION = 'Key properties of the aerosol model'

# --------------------------------------------------------------------
# PROCESS: top level
# --------------------------------------------------------------------
DETAILS = OrderedDict()

DETAILS['toplevel'] = {
    'description': 'Top level key properties in aerosol model',
    'properties': [
        ('scheme_scope', 'ENUM:scheme_scopes', '1.N',
            'Atmospheric domains covered by the aerosol model'),
        ('basic_approximations', 'str', '1.1',
            'Basic approximations made in the aerosol model',),
        ('prognostic_variables_form', 'ENUM:prognostic_vars_types', '1.N',
            'Prognostic variables in the aerosol model'),
        ('number_of_tracers', 'int', '1.1',
            'Number of tracers in the aerosol model'),
        ('family_approach', 'bool', '1.1',
            'Are aerosol calculations generalized into families of species?'),
    ]
}

# --------------------------------------------------------------------
# SUB-PROCESS: timestepping framework
# --------------------------------------------------------------------
DETAILS['timestep_framework'] = {
    'description': 'Physical properties of seawater in ocean',
    'properties' : [
        ('method', 'ENUM:timestepping_methods', '1.1',
            'Mathematical method deployed to solve the time evolution of the prognostic variables'),
        ('split_operator_advection_timestep', 'int', '0.1',
            'Timestep for aerosol advection (in seconds)'),
        ('split_operator_physical_timestep', 'int', '0.1',
            'Timestep for aerosol physics (in seconds).'),
        ('integrated_timestep', 'int', '1.1',
            'Timestep for the aerosol model (in seconds)'),
        ('integrated_scheme_type', 'ENUM:integrated_scheme_types', '1.1',
            'Specify the type of timestep scheme'),
    ],
}

# --------------------------------------------------------------------
# SUB-PROCESS: Meteorological forcings
# --------------------------------------------------------------------
DETAILS['meteorological_forcings'] = {
    'description': '',
    'properties' : [
        ('variables_3D', 'str', '0.1',
            'Three dimensionsal forcing variables, e.g. U, V, W, T, Q, P, conventive mass flux'),
        ('variables_2D', 'str', '0.1',
            'Two dimensionsal forcing variables, e.g. land-sea mask definition'),
        ('frequency', 'int', '0.1',
            'Frequency with which meteological forcings are applied (in seconds).'),
        ]
}

# --------------------------------------------------------------------
# SUB-PROCESS: RESOLUTION: The resolution of the grid.
# --------------------------------------------------------------------
DETAILS['resolution'] = {
    'description': 'Resolution in the aersosol model grid',
    'properties': [
        ('name', 'str', '1.1',
             "This is a string usually used by the modelling group to describe the resolution of this grid, e.g. ORCA025, N512L180, T512L70 etc."),
        ('canonical_horizontal_resolution', 'str', '0.1',
             "Expression quoted for gross comparisons of resolution, eg. 50km or 0.1 degrees etc."),
        ('number_of_horizontal_gridpoints', 'int', '0.1',
             "Total number of horizontal (XY) points (or degrees of freedom) on computational grid."),
        ('number_of_vertical_levels', 'int', '0.1',
             "Number of vertical levels resolved on computational grid."),
        ('is_adaptive_grid', 'bool', '0.1',
             "Default is False. Set true if grid resolution changes during execution."),
    ],
}

# --------------------------------------------------------------------
# SUB-PROCESS: TUNING APPLIED: Any tuning used to optimise the parameters
# --------------------------------------------------------------------
DETAILS['tuning_applied'] = {
    'description': 'Tuning methodology for aerosol model',
    'properties': [
        ('description', 'str', '1.1',
             "General overview description of tuning: explain and motivate the main targets and metrics retained. &"
             "Document the relative weight given to climate performance metrics versus process oriented metrics, &"
             "and on the possible conflicts with parameterization level tuning. In particular describe any struggle &"
             "with a parameter value that required pushing it to its limits to solve a particular model deficiency."),
        ('global_mean_metrics_used', 'str', '0.N',
             "List set of metrics of the global mean state used in tuning model/component"),
        ('regional_metrics_used', 'str', '0.N',
             "List of regional metrics of mean state used in tuning model/component"),
        ('trend_metrics_used', 'str', '0.N',
             "List observed trend metrics used in tuning model/component"),
    ]
}

# --------------------------------------------------------------------
# KEY PROPERTIES: ENUMERATIONS
# --------------------------------------------------------------------
ENUMERATIONS = OrderedDict()

ENUMERATIONS['scheme_scopes'] = {
    'description': 'Atmospheric domains covered by the aerosol model',
    'is_open': True,
    'members': [
        ('troposhere', None),
        ('stratosphere', None),
        ('mesosphere', None),
        ('mesosphere', None),
        ('whole atmosphere', None),
    ]
}

ENUMERATIONS['prognostic_vars_types'] = {
    'description': 'Form of prognostic variables in the aerosol model.',
    'is_open': True,
    'members': [
        ('3D mass/volume ratio for aerosols', None),
        ('3D number concenttration for aerosols', None),
    ]
}

ENUMERATIONS['timestepping_methods'] = {
    'description': 'Mathematical method deployed to solve the time evolution of the prognostic variables.',
    'is_open': True,
    'members': [
        ('Uses atmospheric chemistry time stepping', None),
        ('Specific timestepping (operator splitting)', None),
        ('Specific timestepping (integrated)', None),
    ]
}
     
ENUMERATIONS['integrated_scheme_types'] = {
    'description': 'Specify the type of timestep scheme',
    'is_open': True,
    'members': [
        ('Explicit', None),
        ('Implicit', None),
        ('Semi-implicit', None),
        ('Semi-analytic', None),
        ('Impact solver', None),
        ('Back Euler', None),
        ('Newton Raphson', None),
        ('Rosenbrock', None),
    ]
}



# -*- coding: utf-8 -*-

"""
.. module:: test_type_cim_v1_data_data_object_1.py

   :license: GPL / CeCILL
   :platform: Unix, Windows
   :synopsis: Tests a cim.v1.DataObject instance.

.. moduleauthor:: Earth System Documentation (ES-DOC) <dev@es-doc.org>

"""
import arrow

import pyesdoc.ontologies.cim as cim
import test_utils as tu



# Test type.
DOC_TYPE = cim.v1.DataObject

# Test type display name.
DOC_TYPE_DISPLAY_NAME = "Data Object"

# Test document type.
DOC_TYPE_KEY = DOC_TYPE.type_key + '-1'

# Test representation file.
DOC_FILE = 'cim.1.data.DataObject-1.xml-metafor-cim-v1'

# Test document uid.
DOC_UID = '8e92715a-5176-11e0-9919-00163e9152a5'

# Test document version.
DOC_VERSION = '1'

# Test document creation date.
DOC_DATE = arrow.get('2012-04-23 14:59:07.146365').datetime

# Test document project.
DOC_PROJECT = "CMIP5"

# Test document project.
DOC_INSTITUTE = "MOHC"

# Test document author.
DOC_AUTHOR = "Metafor Questionnaire"


def assert_extension_info(ext):
    """Asserts a document's extension information.

    :param object ext: Document extension information.

    """
    tu.assert_str(ext.display_name, "fossil_fuel_organic_carbon")
    tu.assert_str(ext.description, "2D gridded monthly-mean", True)
    tu.assert_str(ext.full_display_name, "CMIP5 Data Object : MOHC - fossil_fuel_organic_carbon")
    tu.assert_int(ext.summary_fields, 2)
    tu.assert_str(ext.summary_fields[0], "fossil_fuel_organic_carbon")
    tu.assert_str(ext.summary_fields[1], "2D gridded monthly-mean", True)


def _assert_doc_core(doc, is_update):
    """Assert core information."""
    tu.assert_str(doc.acronym, "fossil_fuel_organic_carbon")
    tu.assert_str(doc.data_status, "complete")
    tu.assert_str(doc.description, "2D gridded monthly-mean", True)
    tu.assert_iter(doc.storage, 1)
    storage = doc.storage[0]
    tu.assert_object(storage, cim.v1.DataStorageIp)
    tu.assert_str(storage.format, "ASCII")
    tu.assert_str(storage.file_name, "emissions of organic", True)


def _assert_doc_content(doc, is_update):
    """Assert content information."""
    tu.assert_iter(doc.content, 1, cim.v1.DataContent)
    c = doc.content[0]
    tu.assert_object(c.topic, cim.v1.DataTopic)
    tu.assert_str(c.topic.description, "missing CF name: tendency_of", True)
    tu.assert_str(c.topic.name, "fossil_fuel_organic_carbon")


def assert_doc(doc, is_update=False):
    """Asserts a document.

    :param object doc: Document being tested.
    :param bool is_update: Flag indicating whether document has been updated.

    """
    for assertor in (
        _assert_doc_core,
        _assert_doc_content,
        ):
        assertor(doc, is_update)


def update_doc(doc):
    """Update a document prior to republishing.

    :param object doc: Document being republished.

    """
    pass


def assert_doc_updates(doc):
    """Asserts a document after being updated.

    :param object doc: Document being tested.

    """
    pass


# -*- coding: utf-8 -*-

"""
.. module:: constants.py
   :license: GPL/CeCIL
   :platform: Unix, Windows
   :synopsis: Notebook constants.

.. moduleauthor:: Mark Conway-Greenslade <momipsl@ipsl.jussieu.fr>


"""
# QC status: draft.
QC_STATE_DRAFT = 0

# QC status: in-review.
QC_STATE_IN_REVIEW = 1

# QC status: review.ed
QC_STATE_REVIEWED = 2

# Set of valid QC states.
QC_STATES = {
	QC_STATE_DRAFT,
	QC_STATE_IN_REVIEW,
	QC_STATE_REVIEWED
	}

# Publication state: not ready.
PUBLICATION_STATE_NOT_READY = 0

# Publication state: ready.
PUBLICATION_STATE_READY = 1

# Set of valid publication states.
PUBLICATION_STATES = {
	PUBLICATION_STATE_NOT_READY,
	PUBLICATION_STATE_READY
	}



# -*- coding: utf-8 -*-

"""
.. module:: science_enums.py
   :synopsis: Set of CIM v2 ontology type definitions.

"""


def model_types():
    """Defines a set of gross model classes.

    """
    return {
        'type': 'enum',
        'is_open': False,
        'members': [
            ("Atm Only", "Atmosphere Only"),
            ("Ocean Only", "Ocean Only"),
            ("Regional", "Regional Model"),
            ("ESM", "Earth System Model (Atmosphere, Ocean, Land, Sea-ice and cycles)"),
            ("GCM", "Global Climate Model (Atmosphere, Ocean, no carbon cycle)"),
            ("IGCM", "Intermediate Complexity GCM"),
            ("GCM-MLO", "GCM with mixed layer ocean"),
            ("Mesoscale", "Mesoscale Model"),
            ("Process", "Limited Area process model"),
            ("Planetary", "Non-Earth model")
        ]
    }



# -*- coding: utf-8 -*-

"""
.. module:: cim.v1.typeset_for_shared_package.py

   :license: GPL / CeCILL
   :platform: Unix, Windows
   :synopsis: The set of types of the cim.v1.shared package.

.. moduleauthor:: Earth System Documentation (ES-DOC) <dev@es-doc.org>
.. note:: Code generated using the pyesdoc framework.

"""
import abc
import datetime
import uuid




class Calendar(object):
    """An abstract class within the cim v1 type system.

    Describes a method of calculating a span of dates.

    """
    __metaclass__ = abc.ABCMeta

    def __init__(self):
        """Instance constructor.

        """
        super(Calendar, self).__init__()

        self.description = None                           # unicode (0.1)
        self.length = None                                # int (0.1)
        self.range = None                                 # shared.DateRange (0.1)


class Change(object):
    """A concrete class within the cim v1 type system.

    A description of [a set of] changes applied at a particular time, by a particular party, to a particular unit of metadata.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Change, self).__init__()

        self.author = None                                # shared.ResponsibleParty (0.1)
        self.date = None                                  # datetime.datetime (0.1)
        self.description = None                           # unicode (0.1)
        self.details = []                                 # shared.ChangeProperty (1.N)
        self.name = None                                  # unicode (0.1)
        self.type = None                                  # shared.ChangePropertyType (0.1)


class Citation(object):
    """A concrete class within the cim v1 type system.

    An academic reference to published work.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Citation, self).__init__()

        self.alternative_title = None                     # unicode (0.1)
        self.collective_title = None                      # unicode (0.1)
        self.date = None                                  # datetime.datetime (0.1)
        self.date_type = None                             # unicode (0.1)
        self.location = None                              # unicode (0.1)
        self.organisation = None                          # unicode (0.1)
        self.role = None                                  # unicode (0.1)
        self.title = None                                 # unicode (0.1)
        self.type = None                                  # unicode (0.1)


class Compiler(object):
    """A concrete class within the cim v1 type system.

    A description of a compiler used on a particular platform.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Compiler, self).__init__()

        self.environment_variables = None                 # unicode (0.1)
        self.language = None                              # unicode (0.1)
        self.name = None                                  # unicode (1.1)
        self.options = None                               # unicode (0.1)
        self.type = None                                  # shared.CompilerType (0.1)
        self.version = None                               # unicode (1.1)


class DataSource(object):
    """An abstract class within the cim v1 type system.

    A DataSource can be realised by either a DataObject (file), a DataContent (variable), a Component (model), or a ComponentProperty (variable); all of those can supply data.

    """
    __metaclass__ = abc.ABCMeta

    def __init__(self):
        """Instance constructor.

        """
        super(DataSource, self).__init__()

        self.purpose = None                               # shared.DataPurpose (0.1)


class DateRange(object):
    """An abstract class within the cim v1 type system.

    Creates and returns instance of date_range class.

    """
    __metaclass__ = abc.ABCMeta

    def __init__(self):
        """Instance constructor.

        """
        super(DateRange, self).__init__()

        self.duration = None                              # unicode (0.1)


class DocMetaInfo(object):
    """A concrete class within the cim v1 type system.

    Encapsulates document meta information used by es-doc machinery. Will not normally be
    populated by humans. May duplicate information held in 'visible' metadata.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(DocMetaInfo, self).__init__()

        self.author = None                                # shared.ResponsibleParty (0.1)
        self.create_date = None                           # datetime.datetime (1.1)
        self.drs_keys = []                                # unicode (0.N)
        self.drs_path = None                              # unicode (0.1)
        self.external_ids = []                            # unicode (0.N)
        self.id = None                                    # unicode (1.1)
        self.institute = None                             # unicode (0.1)
        self.project = None                               # unicode (0.1)
        self.sort_key = None                              # unicode (0.1)
        self.source = None                                # unicode (1.1)
        self.source_key = None                            # unicode (0.1)
        self.sub_projects = []                            # unicode (0.N)
        self.type = None                                  # unicode (1.1)
        self.type_display_name = None                     # unicode (0.1)
        self.type_sort_key = None                         # unicode (0.1)
        self.update_date = None                           # datetime.datetime (0.1)
        self.version = None                               # int (1.1)


class DocReference(object):
    """A concrete class within the cim v1 type system.

    A reference to another cim entity.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(DocReference, self).__init__()

        self.canonical_name = None                        # unicode (0.1)
        self.changes = []                                 # shared.Change (0.N)
        self.constraint_vocabulary = None                 # unicode (0.1)
        self.context = None                               # unicode (0.1)
        self.description = None                           # unicode (0.1)
        self.external_id = None                           # unicode (0.1)
        self.id = None                                    # unicode (0.1)
        self.institute = None                             # unicode (0.1)
        self.linkage = None                               # unicode (1.1)
        self.name = None                                  # unicode (1.1)
        self.protocol = None                              # unicode (0.1)
        self.relationship = None                          # unicode (0.1)
        self.type = None                                  # unicode (1.1)
        self.url = None                                   # unicode (0.1)
        self.version = None                               # int (0.1)


class License(object):
    """A concrete class within the cim v1 type system.

    A description of a license restricting access to a unit of data or software.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(License, self).__init__()

        self.contact = None                               # unicode (0.1)
        self.description = None                           # unicode (0.1)
        self.is_unrestricted = None                       # bool (0.1)
        self.name = None                                  # unicode (0.1)


class Machine(object):
    """A concrete class within the cim v1 type system.

    A description of a machine used by a particular platform.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Machine, self).__init__()

        self.cores_per_processor = None                   # int (0.1)
        self.description = None                           # unicode (0.1)
        self.interconnect = None                          # shared.InterconnectType (0.1)
        self.libraries = []                               # unicode (0.N)
        self.location = None                              # unicode (0.1)
        self.maximum_processors = None                    # int (0.1)
        self.name = None                                  # unicode (1.1)
        self.operating_system = None                      # shared.OperatingSystemType (0.1)
        self.processor_type = None                        # shared.ProcessorType (0.1)
        self.system = None                                # unicode (0.1)
        self.type = None                                  # shared.MachineType (0.1)
        self.vendor = None                                # shared.MachineVendorType (0.1)


class MachineCompilerUnit(object):
    """A concrete class within the cim v1 type system.

    Associates a machine with a [set of] compilers.  This is a separate class in case a platform needs to specify more than one machine/compiler pair.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(MachineCompilerUnit, self).__init__()

        self.compilers = []                               # shared.Compiler (0.N)
        self.machine = None                               # shared.Machine (1.1)


class Platform(object):
    """A concrete class within the cim v1 type system.

    A platform is a description of resources used to deploy a component/simulation.  A platform pairs a machine with a (set of) compilers.  There is also a point of contact for the platform.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Platform, self).__init__()

        self.contacts = []                                # shared.ResponsibleParty (0.N)
        self.description = None                           # unicode (0.1)
        self.long_name = None                             # unicode (0.1)
        self.meta = DocMetaInfo()                         # shared.DocMetaInfo (1.1)
        self.short_name = None                            # unicode (1.1)
        self.units = []                                   # shared.MachineCompilerUnit (1.N)


class Property(object):
    """A concrete class within the cim v1 type system.

    A simple name/value pair representing a property of some entity or other.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Property, self).__init__()

        self.name = None                                  # unicode (0.1)
        self.value = None                                 # unicode (0.1)


class Relationship(object):
    """An abstract class within the cim v1 type system.

    A record of a relationship between one document and another. This class is abstract; specific document types must specialise this class for their relationshipTypes to be included in a document's genealogy.

    """
    __metaclass__ = abc.ABCMeta

    def __init__(self):
        """Instance constructor.

        """
        super(Relationship, self).__init__()

        self.description = None                           # unicode (0.1)


class ResponsibleParty(object):
    """A concrete class within the cim v1 type system.

    A person/organsiation responsible for some aspect of a climate science artefact.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ResponsibleParty, self).__init__()

        self.abbreviation = None                          # unicode (0.1)
        self.address = None                               # unicode (0.1)
        self.email = None                                 # unicode (0.1)
        self.individual_name = None                       # unicode (0.1)
        self.organisation_name = None                     # unicode (0.1)
        self.role = None                                  # unicode (0.1)
        self.url = None                                   # unicode (0.1)


class Standard(object):
    """A concrete class within the cim v1 type system.

    Describes a name given to an entity from a recognised standard.  The CIM records the standard and the name.  For example, the standard might be CF and the name might be atmospheric_pressure.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Standard, self).__init__()

        self.description = None                           # unicode (0.1)
        self.name = None                                  # unicode (1.1)
        self.version = None                               # unicode (0.1)


class StandardName(object):
    """A concrete class within the cim v1 type system.

    Describes a name given to an entity from a recognised standard.  The CIM records the standard and the name.  For example, the standard might be CF and the name might be atmospheric_pressure.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(StandardName, self).__init__()

        self.is_open = None                               # bool (1.1)
        self.standards = []                               # shared.Standard (0.N)
        self.value = None                                 # unicode (1.1)


class ChangeProperty(Property):
    """A concrete class within the cim v1 type system.

    A description of a single change applied to a single target.  Every ChangeProperty has a description, and may also have a name from a controlled vocabulary and a value.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ChangeProperty, self).__init__()

        self.description = None                           # unicode (0.1)
        self.id = None                                    # unicode (0.1)


class ClosedDateRange(DateRange):
    """A concrete class within the cim v1 type system.

    A date range with specified start and end points.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(ClosedDateRange, self).__init__()

        self.end = None                                   # datetime.datetime (0.1)
        self.start = None                                 # datetime.datetime (1.1)


class Daily360(Calendar):
    """A concrete class within the cim v1 type system.

    Creates and returns instance of daily_360 class.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(Daily360, self).__init__()



class OpenDateRange(DateRange):
    """A concrete class within the cim v1 type system.

    A date range without a specified start and/or end point.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(OpenDateRange, self).__init__()

        self.end = None                                   # datetime.datetime (0.1)
        self.start = None                                 # datetime.datetime (0.1)


class PerpetualPeriod(Calendar):
    """A concrete class within the cim v1 type system.

    Creates and returns instance of perpetual_period class.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(PerpetualPeriod, self).__init__()



class RealCalendar(Calendar):
    """A concrete class within the cim v1 type system.

    Creates and returns instance of real_calendar class.

    """
    def __init__(self):
        """Instance constructor.

        """
        super(RealCalendar, self).__init__()



class ChangePropertyType(object):
    """An enumeration within the cim v1 type system.

    Creates and returns instance of change_property_type enum.
    """
    is_open = False
    members = [
        "InputMod",
        "ModelMod",
        "Decrement",
        "Increment",
        "Redistribution",
        "Replacement",
        "ParameterChange",
        "CodeChange",
        "AncillaryFile",
        "BoundaryCondition",
        "InitialCondition",
        "Unused"
        ]
    descriptions = [
        "None",
        "None",
        "None",
        "None",
        "None",
        "None",
        "A specific type of ModelMod",
        "A specific type of ModelMod",
        "A specific type of ModelMod",
        "A specific type of ModelMod",
        "A specific type of ModelMod",
        "None"
        ]


class CompilerType(object):
    """An enumeration within the cim v1 type system.

    Creates and returns instance of compiler_type enum.
    """
    is_open = True
    members = []
    descriptions = []


class DataPurpose(object):
    """An enumeration within the cim v1 type system.

    Purpose of the data - i.e. ancillaryFile, boundaryCondition or initialCondition.
    """
    is_open = False
    members = [
        "ancillaryFile",
        "boundaryCondition",
        "initialCondition"
        ]
    descriptions = [
        "None",
        "None",
        "None"
        ]


class InterconnectType(object):
    """An enumeration within the cim v1 type system.

    A list of connectors between machines.
    """
    is_open = True
    members = []
    descriptions = []


class MachineType(object):
    """An enumeration within the cim v1 type system.

    A list of known machines.
    """
    is_open = False
    members = [
        "Parallel",
        "Vector",
        "Beowulf"
        ]
    descriptions = [
        "None",
        "None",
        "None"
        ]


class MachineVendorType(object):
    """An enumeration within the cim v1 type system.

    A list of organisations that create machines.
    """
    is_open = True
    members = []
    descriptions = []


class OperatingSystemType(object):
    """An enumeration within the cim v1 type system.

    A list of common operating systems.
    """
    is_open = True
    members = []
    descriptions = []


class ProcessorType(object):
    """An enumeration within the cim v1 type system.

    A list of known cpu's.
    """
    is_open = True
    members = []
    descriptions = []


class UnitType(object):
    """An enumeration within the cim v1 type system.

    A list of scientific units.
    """
    is_open = True
    members = []
    descriptions = []


